"""Backward-compatible facade for the renamed DeepthinkEngine."""

from analogai.deepthink.deepthink_engine import DeepthinkEngine


class Deepthinker(DeepthinkEngine):
	"""Facade class to preserve the public API surface."""


DeepThinker = Deepthinker
Thinker = Deepthinker



